import java.applet.*;
import java.awt.*;

public class EX15B extends Applet implements Runnable
{
	protected final int ImageIdBase = 1;
	protected final int ImageCount = 20;
	protected Image Images[];
	protected int LoadedCount;
	protected int CurrentImageId = ImageIdBase;
	protected MediaTracker ImageTracker;
	protected Thread ImageThread;
	protected Label ProgressMessage = new Label();


	public void init()
	{
        // If you use a ResourceWizard-generated "control creator" class to
        // arrange controls in your applet, you may want to call its
        // CreateControls() method from within this method. Remove the following
        // call to resize() before adding the call to CreateControls();
        // CreateControls() does its own resizing.
        //----------------------------------------------------------------------
		resize(320, 240);

		add(ProgressMessage);

		// create a tracker to monitor image loading
		ImageTracker = new MediaTracker(this);

		// create a place to put the images
		Images = new Image[ImageCount];

		// add the images to be tracked
		for (int id = ImageIdBase; id <= ImageIdBase + ImageCount - 1;
				id++) {
			Images[id - ImageIdBase] = getImage(getCodeBase(),
					"images\\Banner" + Integer.toString(id) + ".gif");
			ImageTracker.addImage(Images[id - ImageIdBase], id);
		}
	}

	public void paint(Graphics g)
	{
		// if haven't loaded all of the images, then don't do anything
		if (LoadedCount >= ImageCount) {
			int currentId = CurrentImageId;

			for (int index = 0; index < 6; index++) {
				// assume images have a width of 50 
				g.drawImage(Images[currentId - ImageIdBase], 
						index * 50, 0, this);

				currentId = GetNextId(currentId);
			}
		}
	}

	public void start()
	{
		if (ImageThread == null) {
			ImageThread = new Thread(this);
			ImageThread.start();
		}
	}
	
	public void stop()
	{
		if (ImageThread != null) {
			ImageThread.stop();
			ImageThread = null;
		}
	}

	public void run()
	{
		int id = ImageIdBase;

		while (id <= ImageIdBase + ImageCount - 1) {
			if ((ImageTracker.statusID(id, true) & MediaTracker.COMPLETE)
					== MediaTracker.COMPLETE) {
				IncrementImageCount();

				id++;
			}
			else {
				try {
					Thread.sleep(250);
				} catch (InterruptedException e) {}
			}
		}

		ProgressMessage.hide();			// don't show progress any more

		while (true) {
			repaint();

			try {
				Thread.sleep(750);
			} catch (InterruptedException e) {}

			CurrentImageId = GetNextId(CurrentImageId);
		}
	}

	protected synchronized void IncrementImageCount()
	{
		LoadedCount++;

		// inform the user what is happening
		ProgressMessage.setText("Loaded " + 
				Integer.toString(LoadedCount) + " of the " +
				Integer.toString(ImageCount) + " images...");
		ProgressMessage.resize(ProgressMessage.preferredSize());

		// tell the layout manager to do its job
		validate();
	}

	protected int GetNextId(int currentId)
	{
		int nextId = currentId + 1;

		if (nextId >= ImageCount + ImageIdBase)
			nextId = ImageIdBase;

		return nextId;
	}
}
